@@ -242,8 +242,10 @@ static near_bool_t snep_read(int client_fd,
 		return FALSE;
 
 	snep_data->nfc_data = g_try_malloc0(ndef_length + TLV_SIZE);
-	if (snep_data->nfc_data == NULL)
+	if (snep_data->nfc_data == NULL) {
+		g_free(snep_data);
 		return FALSE;
+	}
 
 	snep_data->nfc_data_length = ndef_length;
 	snep_data->nfc_data_ptr = snep_data->nfc_data;
@@ -265,7 +267,6 @@ static near_bool_t snep_read(int client_fd,
 		return FALSE;
 	case SNEP_REQ_PUT:
 		return snep_read_ndef(client_fd, snep_data);
-		break;
 	}
 
 	return FALSE;
@@ -508,15 +509,10 @@ static int snep_push(int fd, uint32_t adapter_idx, uint32_t target_idx,
 
 	g_free(fragment->data);
 	g_free(fragment);
-	g_free(ndef->data);
-	g_free(ndef);
 
 	return 0;
 
 error:
-	g_free(ndef->data);
-	g_free(ndef);
-
 	free_snep_push_data(req, err);
 
 	return err;
@@ -524,7 +520,7 @@ error:
 
 struct near_p2p_driver snep_driver = {
 	.name = "SNEP",
-	.service_name = "urn:nfc:sn:snep",
+	.service_name = NEAR_DEVICE_SN_SNEP,
 	.read = snep_read,
 	.push = snep_push,
 	.close = snep_close,
